<?php

class HappyForms_Message_Blocklist {

	private static $instance;
	private static $hooked = false;

	public $save_action = 'happyforms_save_message_blocklist';
	public $save_nonce = 'happyforms-validation-message-blocklist';
	public $option = 'happyforms_blocklist';

	public static function instance() {
		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}

		self::$instance->hook();

		return self::$instance;
	}

	public function hook() {
		if ( self::$hooked ) {
			return;
		}

		self::$hooked = true;

		add_filter( 'happyforms_meta_fields', array( $this, 'meta_fields' ) );
		add_filter( 'happyforms_setup_controls', array( $this, 'setup_controls' ) );
		add_filter( 'happyforms_get_form_data', array( $this, 'transition_per_form_blocklist'), 99 );
	}

	public function get_fields() {
		$fields = array(
			'block_emails' => array(
				'default' => 0,
				'sanitize' => 'happyforms_sanitize_checkbox',
			),
			'blocklist_emails' => array(
				'default' => '',
				'sanitize' => array( $this, 'sanitize_email_keys' ),
			),
			'block_language' => array(
				'default' => 0,
				'sanitize' => 'happyforms_sanitize_checkbox',
			),
			'blocklist_language' => array(
				'default' => '',
				'sanitize' => array( $this, 'sanitize_language_keys' ),
			),
			// TODO delete once handler for global blocklist has been complete removed.
			'per_form_blocklist' => array(
				'default' => 0,
				'sanitize' => 'sanitize_text_field',
			),
		);

		return $fields;
	}

	public function setup_controls( $controls ) {
		$setup_controls = array(
			6000 => array(
				'type' => 'checkbox',
				'label' => __( 'Block these email addresses, email domains and email partials', 'happyforms' ),
				'field' => 'block_emails',
			),
			6001 => array(
				'type' => 'group_start',
				'trigger' => 'block_emails'
			),
			6002 => array(
				'type' => 'textarea',
				'label' => __( '', 'happyforms' ),
				'field' => 'blocklist_emails',
				'autocomplete' => 'off',
				'description' => __( 'One per line. Not case sensitive. Blank spaces disallowed', 'happyforms' ),
			),
			6004 => array(
				'type' => 'group_end'
			),
			6005 => array(
				'type' => 'checkbox',
				'label' => __( 'Block these words, phrases and letters', 'happyforms' ),
				'field' => 'block_language',
			),
			6006 => array(
				'type' => 'group_start',
				'trigger' => 'block_language'
			),
			6007 => array(
				'type' => 'textarea',
				'label' => __( '', 'happyforms' ),
				'field' => 'blocklist_language',
				'autocomplete' => 'off',
				'description' => __( 'One per line. Not case sensitive. Punctuation, numbers and alt characters disallowed.', 'happyforms' ),
			),
			6008 => array(
				'type' => 'group_end'
			),
		);

		$controls = happyforms_safe_array_merge( $controls, $setup_controls );

		return $controls;
	}

	public function meta_fields( $fields ) {
		$fields = array_merge( $fields, $this->get_fields() );

		return $fields;
	}

	public function transition_per_form_blocklist( $form ) {
		if( 0 == $form['ID'] ) {
			$form['per_form_blocklist'] = 1;
			return $form;
		}

		if( 0 == $form['per_form_blocklist'] ) {
			$blocklist = $this->read();

			if( 1 == $blocklist['block_emails'] ) {
				$form['block_emails'] = 1;
				$form['blocklist_emails'] = $blocklist['emails'];
			}

			if( 1 == $blocklist['block_language'] ) {
				$form['block_language'] = 1;
				$form['blocklist_language'] = $blocklist['language'];
			}

			$form['per_form_blocklist'] = 1;
		}

		return $form;

	}

	// TODO delete once handler for global blocklist has been complete removed.
	public function read( $as_array = false ) {
		$blocklist = get_option( $this->option, '' );
		$defaults = array(
			'block_language' => 0,
			'language' => '',
			'block_emails' => 0,
			'emails' => '',
		);

		$blocklist = wp_parse_args( $blocklist, $defaults );
		$blocklist = array_map( 'wp_unslash', $blocklist );

		return $blocklist;
	}

	public function sanitize_email_keys( $emails ) {
		$lines = explode( "\n", $emails );
		$lines = array_map( 'trim', $lines );
		$lines = array_filter( $lines, function( $line ) {
			return ( ! preg_match( '/\s/', $line ) );
		} );
		$lines = array_filter( $lines );
		$emails = implode( "\n", $lines );

		return $emails;
	}

	public function sanitize_language_keys( $language ) {
		$lines = explode( "\n", $language );
		$lines = array_map( 'trim', $lines );
		$lines = array_filter( $lines, function( $line ) {
			return ( ! preg_match( '/[^\p{L} ]/u', $line ) );
		} );
		$lines = array_map( function( $line ) {
			$line = preg_replace( '/\s+/u', ' ', $line );
			return $line;
		}, $lines );
		$lines = array_filter( $lines );
		$language = implode( "\n", $lines );

		return $language;
	}

	public function includes_payment( $form, $request ) {
		$payment_part = happyforms_get_form_controller()->get_first_part_by_type( $form, 'payments' );

		if ( ! $payment_part ) {
			return;
		}

		$payment_part_id = $payment_part['id'];
		$payment_part_name = happyforms_get_part_name( $payment_part, $form );
		$payment_part_value = $request[$payment_part_name];
		$price = isset( $payment_part_value['price'] ) ? $payment_part_value['price'] : '';
		$price = floatval( $price );
		$includes_payment = $price > 0;

		return $includes_payment;
	}

	public function check_value_email( $value, $keys ) {
		$value = trim( $value );

		foreach( $keys as $key ) {
			$key = preg_quote( $key, '#' );
			$pattern = "#$key#i";
			
			if ( preg_match( $pattern, $value ) ) {
				return true;
			}
		}

		return false;
	}

	public function check_value_text( $value, $keys ) {
		$values = is_array( $value ) ? $value : array( $value );

		$values = array_map( function( $value ) {
			$value = preg_replace( '/<[^>]*>/u', ' ', $value );
			$value = preg_replace( '/[^\p{L} ]/u', '', $value );
			$value = preg_replace( '/\s+/u', ' ', $value );
			$value = trim( $value );

			return $value;
		}, $values );
		$values = array_filter( $values );
		
		foreach( $keys as $key ) {
			foreach( $values as $value ) {
				$pattern = '/\b' . $key . '\b/miu';
				
				if ( preg_match( $pattern, $value ) ) {
					return true;
				}
			}
		}

		return false;
	}

	public function check_emails_value( $part, $form, $message, $blocklist ) {
		$part_id = $part['id'];
		$part_type = $part['type'];

		if ( ! isset( $message['parts'][$part_id] ) ) {
			return false;
		}

		switch( $part_type ) {
			case 'email':
				return $this->check_value_email( $message['parts'][$part_id], $blocklist );
				break;
		}
	}

	public function check_language_value( $part, $form, $message, $blocklist ) {
		$part_id = $part['id'];
		$part_type = $part['type'];

		if ( ! isset( $message['parts'][$part_id] ) ) {
			return false;
		}

		switch( $part_type ) {
			case 'single_line_text':
			case 'multi_line_text':
			case 'rich_text':
				return $this->check_value_text( $message['parts'][$part_id], $blocklist );
				break;
			case 'narrative':
				$part_name = happyforms_get_part_name( $part, $form );
				return $this->check_value_text( $message['request'][$part_name], $blocklist );
				break;
			case 'signature':
				$part_name = happyforms_get_part_name( $part, $form );
				return $this->check_value_text( $message['request'][$part_name]['signature'], $blocklist );
				break;
			case 'select':
			case 'radio':
			case 'checkbox':
				if ( $part['other_option'] ) {
					$part_name = happyforms_get_part_name( $part, $form );
					$part_value = $message['request'][$part_name];
					$part_value = is_array( $part_value ) ? end( $part_value ) : $part_value;
					return $this->check_value_text( $part_value, $blocklist['language'] );
				}

				break;
		}
	}

	public function check_message( $message, $form, $request ) {
		$is_spam = false;

		if ( $this->includes_payment( $form, $request ) ) {
			return $is_spam;
		}

		$block_emails = $form['block_emails'];
		$block_language = $form['block_language'];
		$blocklist_emails = '';
		$blocklist_language = '';

		if ( ( 0 == $block_emails && 0 == $block_language ) ||
			( empty( $form['blocklist_emails'] ) && empty( $form['blocklist_language'] ) ) ) {
			return $is_spam;
		}

		if( 1 == $block_emails ) {
			$blocklist_emails = $form['blocklist_emails'];
			$blocklist_emails = explode( "\n", $blocklist_emails );
		}

		if( 1 == $block_language ) {
			$blocklist_language = $form['blocklist_language'];
			$blocklist_language = explode( "\n", $blocklist_language );
		}

		foreach( $form['parts'] as $part ) {
			if( 1 == $block_emails ) {
				$is_spam = $this->check_emails_value( $part, $form, $message, $blocklist_emails );

				if ( $is_spam ) {
					break;
				}
			}

			if( 1 == $block_language ) {
				$is_spam = $this->check_language_value( $part, $form, $message, $blocklist_language );

				if ( $is_spam ) {
					break;
				}
			}
		}

		if ( $is_spam ) {
			happyforms_update_meta( $message['ID'], 'read', 2 );
			happyforms_get_message_controller()->update_counters();
		}

		return $is_spam;
	}

}

if ( ! function_exists( 'happyforms_get_message_blocklist' ) ):

function happyforms_get_message_blocklist() {
	return HappyForms_Message_Blocklist::instance();
}

endif;

happyforms_get_message_blocklist();
