<?php

class HappyForms_Service_Email extends HappyForms_Service {

	private static $instance;
	private static $hooked = false;

	public $id = 'email';
	public $active_service_option_name = '';
	public $active_service = false;

	public function __construct() {
		$this->label = __( 'Email', 'happyforms' );
		$this->active_service_option_name = "_happyforms_{$this->id}_service_active";
	}

	public static function instance() {
		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	public function hook() {
		if ( self::$hooked ) {
			return;
		}

		self::$hooked = true;

		add_filter( 'happyforms_get_form_data', array( $this, 'set_form_data'), 99 );
	}

	public function set_form_data( $form ){
		if ( 0 === $form['ID'] ) {
			return $form;
		}

		$service = get_option( $this->active_service_option_name, '' );
		if ( '' === $service ) {
			return $form;
		}

		$service_enable_field = 'enable_';
		switch ( $service ) {
			case 'active-campaign':
				$service_enable_field .= 'active_campaign';
				break;
			case 'constant-contact':
				$service_enable_field .= 'constant_contact';
				break;
			default:
				$service_enable_field .= $service;
				break;
		}

		if ( empty( $form[ $service_enable_field ] ) || 0 === $form[ $service_enable_field ] ) {
			return $form;
		}

		$form[ 'active_email_service' ] = $service;

		return $form;
	}

	public function set_active_service( $service_id ) {
		update_option( $this->active_service_option_name, $service_id );
	}

	public function get_active_service() {
		$service = get_option( $this->active_service_option_name, false );

		if ( ! empty( $service ) ) {
			$service = happyforms_get_integrations()->get_service( $service );
		}

		return $service;
	}

}

if ( ! function_exists( 'happyforms_deprecated_email_service' ) ):

function happyforms_deprecated_email_service() {
	$instance = HappyForms_Service_Email::instance();
	$instance->hook();

	return $instance;
}

endif;

happyforms_deprecated_email_service();
