<?php

if ( ! class_exists( 'TTF_Product' ) ) :

class TTF_Product {

	public $id = '';
	public $name = '';
	public $slug = '';
	public $version = '';
	public $plan = '';
	public $type = '';
	public $option = '_ttf_product_updater_key_%s';
	public $event_check = '_ttf_product_updater_license_check_%s';

	public $url_oauth = '%s/oauth';
	public $url_request_key = '%s/wp-json/updates/request';
	public $url_register = '%s/wp-json/updates/register';
	public $url_feed = '%s/wp-json/updates/feed';
	public $url_package = '%s/wp-json/updates/package/%s';
	public $url_download = '%s/wp-json/updates/download/%s';
	public $url_license_check = '%s/wp-json/updates/check';

	public function __construct( $args = array() ) {
		$args = wp_parse_args( $args, $this->get_defaults() );

		extract( $args );

		$this->id = $id;
		$this->name = $name;
		$this->slug = $slug;
		$this->version = $version;
		$this->plan = $plan;
		$this->type = $type;
		$this->option = sprintf( $this->option, $this->id );
		$this->event_check = sprintf( $this->event_check, $this->id );

		$this->url_oauth = sprintf( $this->url_oauth, $url );
		$this->url_request_key = sprintf( $this->url_request_key, $url );
		$this->url_register = sprintf( $this->url_register, $url );
		$this->url_feed = sprintf( $this->url_feed, $url );
		$this->url_package = sprintf( $this->url_package, $url, $this->plan );
		$this->url_download = sprintf( $this->url_download, $url, $this->id );
		$this->url_license_check = sprintf( $this->url_license_check, $url );
	}

	public function get_defaults() {
		$defaults = array(
			'id' => '',
			'name' => '',
			'slug' => '',
			'version' => '',
			'plan' => '',
			'type' => '',
			'url' => '',
		);

		return $defaults;
	}

}

endif;

if ( ! class_exists( 'TTF_Product_Updater' ) ) :

class TTF_Product_Updater {

	/**
	 *
	 * Ajax actions for registering/deregistering
	 * a license key from the dashboard.
	 *
	 */
	private $action_skip = 'ttf-product-updater-skip';
	private $action_request_key = 'ttf-product-updater-request-key';
	private $action_cancel = 'ttf-product-updater-cancel';
	private $action_authorize = 'ttf-product-updater-authorize';
	private $action_deauthorize = 'ttf-product-updater-deauthorize';

	/**
	 *
	 * Holds product metadata.
	 *
	 */
	public $product;

	public function __construct( $args = array() ) {
		$this->product = new TTF_Product( $args );

		// Dashboard license widget actions
		add_action( 'wp_ajax_' . $this->action_skip, array( $this, 'skip_email' ) );
		add_action( 'wp_ajax_' . $this->action_request_key, array( $this, 'request_key' ) );
		add_action( 'wp_ajax_' . $this->action_cancel, array( $this, 'cancel' ) );
		add_action( 'wp_ajax_' . $this->action_authorize, array( $this, 'authorize' ) );
		add_action( 'wp_ajax_' . $this->action_deauthorize, array( $this, 'deauthorize' ) );

		// Rendering
		add_action( 'admin_notices', array( $this, 'admin_notices' ) );
		add_action( 'admin_head', array( $this, 'print_header_styles' ), PHP_INT_MAX );

		// Remote updates handling
		add_filter( 'pre_set_site_transient_update_plugins', array( $this, 'pre_set_site_transient_update_plugins' ) );
		add_filter( 'pre_set_site_transient_update_themes', array( $this, 'pre_set_site_transient_update_themes' ) );
		add_filter( 'plugins_api', array( $this, 'plugins_api' ), 10, 3 );
		add_filter( 'themes_api', array( $this, 'themes_api' ), 10, 3 );
		add_filter( 'upgrader_package_options', [ $this, 'upgrader_package_options' ] );
		add_filter( 'upgrader_pre_download', array( $this, 'upgrader_pre_download' ), 10, 3 );
		add_filter( 'http_request_args', array( $this, 'http_request_args' ), 20, 2 );

		// License checks
		if ( ! wp_next_scheduled ( $this->product->event_check ) ) {
			wp_schedule_event( time(), 'daily', $this->product->event_check );
		}

		add_action( $this->product->event_check, array( $this, 'license_check' ) );
	}

	public function get_license_key() {
		return get_option( $this->product->option, false );
	}

	public function is_happyforms_product() {
		$is_happyforms_product = strpos( $this->product->id, 'happyforms' ) !== false;

		return $is_happyforms_product;
	}

	public function is_themefoundry_product() {
		$is_themefoundry_product = ! $this->is_happyforms_product();

		return $is_themefoundry_product;
	}

	public function has_old_happyforms_key() {
		$happyforms_key = get_option( 'ttf_updates_key_happyforms', true );
		$has_old_happyforms_key = $happyforms_key !== true;

		return $has_old_happyforms_key;
	}

	public function has_old_themefoundry_key() {
		$themefoundry_key = get_option( 'ttf-api-key', true );
		$has_old_themefoundry_key = $themefoundry_key !== true;

		return $has_old_themefoundry_key;
	}

	/**
	 *
	 * Dashboard page methods
	 *
	 */
	public function skip_email() {
		$this->render_dashboard_widget( $this->action_authorize );

		exit();
	}

	public function request_key() {
		if ( ! check_admin_referer( $this->action_request_key ) ) {
			return;
		}

		if ( ! isset( $_POST['product_plan'] ) || empty( $_POST['product_plan'] ) ) {
			return;
		}

		$product_plan = $_POST['product_plan'];

		if ( $this->product->plan !== $product_plan ) {
			return;
		}

		$email = isset( $_POST['email'] ) ? trim( $_POST['email'] ) : '';

		// An email address must be supplied.
		if ( empty( $email ) ) {
			add_action( 'ttf_updater_notices', function( $action ) {
				if ( $this->action_request_key !== $action ) {
					return;
				} ?>
				<div class="notice notice-error">
					<p><?php _e( 'Oops! Please enter an email address.', 'ttf-product-updater' ); ?></p>
				</div>
			<?php } );

			$this->render_dashboard_widget();
			exit();
		}

		$response = wp_remote_post( $this->product->url_request_key, array(
			'headers' => array(
				'X-TTF-PRODUCT-PLAN' => $product_plan,
				'X-TTF-PRODUCT-VERSION' => $this->product->version,
				'X-TTF-SITE-URL' => get_site_url(),
			),
			'body' => array(
				'email' => $email,
			),
			'timeout' => 5,
		) );

		$body = wp_remote_retrieve_body( $response );

		// Catch-all, generic error handler.
		if ( ! $body ) {
			add_action( 'ttf_updater_notices', function( $action ) {
				if ( $this->action_request_key !== $action ) {
					return;
				} ?>
				<div class="notice notice-error">
					<p><?php _e( 'An unexpected error occurred.', 'ttf-product-updater' ); ?></p>
				</div>
			<?php } );

			$this->render_dashboard_widget();
			exit();
		}

		$body = json_decode( $body );

		// Catch-all, generic error handler.
		if ( 'success' !== $body->code ) {
			add_action( 'ttf_updater_notices', function( $action ) {
				if ( $this->action_request_key !== $action ) {
					return;
				} ?>
				<div class="notice notice-error">
					<p><?php _e( 'An unexpected error occurred.', 'ttf-product-updater' ); ?></p>
				</div>
			<?php } );

			$this->render_dashboard_widget();
			exit();
		}

		// Render the widget with a success notice.
		add_action( 'ttf_updater_notices', function( $action ) {
				if ( $this->action_authorize !== $action ) {
					return;
				} ?>
			<div class="notice notice-success">
				<p><?php _e( 'Registration key sent. Please check your inbox.', 'ttf-product-updater' ); ?></p>
			</div>
		<?php } );

		$this->render_dashboard_widget( $this->action_authorize, $email );
		exit();
	}

	public function cancel() {
		$this->render_dashboard_widget();
		exit();
	}

	public function authorize() {
		if ( ! check_admin_referer( $this->action_authorize ) ) {
			return;
		}

		if ( ! isset( $_POST['product_plan'] ) || empty( $_POST['product_plan'] ) ) {
			return;
		}

		$product_plan = $_POST['product_plan'];

		if ( $this->product->plan !== $product_plan ) {
			return;
		}

		$email = isset( $_POST['email'] ) ? trim( $_POST['email'] ) : '';

		// A license key must be supplied.
		if ( ! isset( $_POST['license_key'] ) || empty( $_POST['license_key'] ) ) {
			add_action( 'ttf_updater_notices', function( $action ) {
				if ( $this->action_authorize !== $action ) {
					return;
				} ?>
				<div class="notice notice-error">
					<p><?php _e( 'Oops! Please enter a registration key.', 'ttf-product-updater' ); ?></p>
				</div>
			<?php } );

			$this->render_dashboard_widget( $this->action_authorize, $email );
			exit();
		}

		$license_key = $_POST['license_key'];

		$response = wp_remote_post( $this->product->url_register, array(
			'headers' => array(
				'X-TTF-LICENSE-KEY' => $license_key,
				'X-TTF-PRODUCT-PLAN' => $product_plan,
				'X-TTF-PRODUCT-VERSION' => $this->product->version,
				'X-TTF-SITE-URL' => get_site_url(),
			),
			'body' => array(
				'email' => $email,
			),
			'timeout' => 5,
		) );

		$body = wp_remote_retrieve_body( $response );

		// Catch-all, generic error handler.
		if ( ! $body ) {
			add_action( 'ttf_updater_notices', function( $action ) {
				if ( $this->action_authorize !== $action ) {
					return;
				} ?>
				<div class="notice notice-error">
					<p><?php _e( 'An unexpected error occurred.', 'ttf-product-updater' ); ?></p>
				</div>
			<?php } );

			$this->render_dashboard_widget( $this->action_authorize, $email );
			exit();
		}

		$body = json_decode( $body );

		// License key must be valid.
		if ( 'success' !== $body->code ) {
			add_action( 'ttf_updater_notices', function( $action ) {
				if ( $this->action_authorize !== $action ) {
					return;
				} ?>
				<div class="notice notice-error">
					<p><?php _e( 'Oops! Either your key is incorrect or your membership is deactivated.', 'ttf-product-updater' ); ?></p>
				</div>
			<?php } );

			$this->render_dashboard_widget( $this->action_authorize, $email );
			exit();
		}

		// Remove deprecated license keys
		if ( $this->is_happyforms_product() && $this->has_old_happyforms_key() ) {
			delete_option( 'ttf_updates_key_happyforms' );
		}

		if ( $this->is_themefoundry_product() && $this->has_old_themefoundry_key() ) {
			delete_option( 'ttf-api-key' );
		}

		// Store the license key.
		update_option( $this->product->option, $license_key );

		// Render the widget with a success notice.
		add_action( 'ttf_updater_notices', function( $action ) {
			if ( $this->action_authorize !== $action ) {
				return;
			} ?>
			<div class="notice notice-success">
				<p><?php _e( 'Successfully registered.', 'ttf-product-updater' ); ?></p>
			</div>
		<?php } );

		$this->render_dashboard_widget();
		exit();
	}

	public function deauthorize() {
		if ( ! check_admin_referer( $this->action_deauthorize ) ) {
			return;
		}

		if ( ! isset( $_POST['product_plan'] ) || empty( $_POST['product_plan'] ) ) {
			return;
		}

		$product_plan = $_POST['product_plan'];

		if ( $this->product->plan !== $product_plan ) {
			return;
		}

		// Delete the license key.
		delete_option( $this->product->option );

		// Render the widget without notices.
		$this->render_dashboard_widget();
		exit();
	}

	public function render_dashboard_widget( $action = '', $email = '' ) {
		add_action( 'admin_print_footer_scripts', [ $this, 'print_footer_scripts' ], PHP_INT_MAX );
		?>
		<div id="ttf-product-updater-form-container">
			<?php if ( ! $this->get_license_key() ) : ?>
				<?php if ( '' === $action ) : ?>
					<form action="" method="post" class="ttf-product-updater-form">
						<?php do_action( 'ttf_updater_notices', $this->action_request_key ); ?>
						<?php wp_nonce_field( $this->action_request_key ); ?>
						<input type="hidden" name="action" value="<?php echo $this->action_request_key; ?>" />
						<input type="hidden" name="product_plan" value="<?php echo $this->product->plan; ?>" />
						<?php $message = apply_filters(
							'ttf_product_updater_membership_required_message',
							__( 'You need an active membership', 'ttf-product-updater' ),
							$this->product->id
						); ?>
						<p><?php echo $message; ?>.</p>
						<p>
							<label for="email"><?php _e( 'Email address', 'ttf-product-updater' ); ?>:</label>
							<input type="email" name="email" placeholder="<?php _e( 'Must be the same address you used to sign up', 'ttf-product-updater' ); ?>" value="" class="large-text" />
						</p>
						<button type="submit" class="button button-primary"><?php _e( 'Send Registration Key', 'ttf-product-updater' ); ?></button>
						<p style="margin-bottom: 0;">
							<?php _e( 'Already got your registration key?', 'ttf-product-updater' ); ?> <a href="#" class="skip"><?php _e( 'Jump ahead' , 'ttf-product-updater' ); ?></a>.
						</p>
					</form>
					<?php elseif ( $this->action_authorize === $action ) : ?>
					<form action="" method="post" class="ttf-product-updater-form">
						<?php do_action( 'ttf_updater_notices', $this->action_authorize ); ?>
						<?php wp_nonce_field( $this->action_authorize ); ?>
						<input type="hidden" name="action" value="<?php echo $this->action_authorize; ?>" />
						<input type="hidden" name="product_plan" value="<?php echo $this->product->plan; ?>" />
						<input type="hidden" name="email" value="<?php echo $email; ?>" />
						<p>
							<label for="license_key"><?php _e( 'Registration key', 'ttf-product-updater' ); ?>:</label>
							<span class="ttf-updater-password-input">
								<input type="password" name="license_key" placeholder="" value="" class="large-text" />
								<button type="button" class="button ttf-updater-hide-pw hide-if-no-js" data-toggle="0" aria-label="<?php _e( 'Show credentials', 'ttf-product-updater' ); ?>" data-label-show="<?php _e( 'Show credentials', 'ttf-product-updater' ); ?>" data-label-hide="<?php _e( 'Hide credentials', 'ttf-product-updater' ); ?>">
									<span class="dashicons dashicons-visibility" aria-hidden="true"></span>
								</button>
							</span>
						</p>
						<p style="margin-bottom: 0;">
							<button type="button" class="button cancel"><?php _e( 'Cancel', 'ttf-product-updater' ); ?></button>
							<button type="submit" class="button button-primary"><?php _e( 'Register', 'ttf-product-updater' ); ?></button>
						</p>
					</form>
					<?php if ( ! empty( $email ) ) : ?>
					<form action="" method="post" class="ttf-product-updater-form" style="margin-top: 5px;">
						<?php wp_nonce_field( $this->action_request_key ); ?>
						<input type="hidden" name="action" value="<?php echo $this->action_request_key; ?>" />
						<input type="hidden" name="product_plan" value="<?php echo $this->product->plan; ?>" />
						<input type="hidden" name="email" value="<?php echo $email; ?>" />
						<p style="margin: 5px 0 0;">
							<?php _e( 'Still no email?', 'ttf-product-updater' ); ?> <button type="submit" class="button-link" style="text-decoration: underline;"><?php _e( 'Resend', 'ttf-product-updater' ); ?></button>.
						</p>
					</form>
					<?php endif; ?>
				<?php endif; ?>
			<?php else : ?>
			<form action="" method="post" class="ttf-product-updater-form">
				<?php do_action( 'ttf_updater_notices', $this->action_deauthorize ); ?>
				<?php wp_nonce_field( $this->action_deauthorize ); ?>
				<input type="hidden" name="action" value="<?php echo $this->action_deauthorize; ?>" />
				<input type="hidden" name="product_plan" value="<?php echo $this->product->plan; ?>" />
				<p><?php _e( 'Good for you, you\'ve registered.', 'ttf-product-updater' ); ?></p>
				<button type="submit" class="button button-primary"><?php _e( 'Register Again', 'ttf-product-updater' ); ?></button>
			</form>
			<?php endif; ?>
		</div>
		<?php
	}

	public function admin_notices() {
		$screen = get_current_screen();

		if ( $screen && in_array( $screen->id, array( 'update', 'update-core' ) ) ) {
			return;
		}

		$display_notice = apply_filters( 'ttf_product_updater_show_transition_notice', true, $this->product->id );

		if ( ! $display_notice ) {
			return;
		}

		if ( $this->is_happyforms_product() && ! $this->has_old_happyforms_key() ) {
			return;
		}

		if ( $this->is_themefoundry_product() && ! $this->has_old_themefoundry_key() ) {
			return;
		}

		$message = apply_filters(
			'ttf_product_updater_transition_message',
			sprintf(
				__( 'Heads up! %s remote updates have been deactivated. To activate again, please register.', 'ttf-product-updater' ),
				$this->product->name
			),
			$this->product->id
		);
		?>
		<div class="notice notice-warning">
			<p><?php echo $message; ?></p>
		</div>
		<?php
	}

	/**
	 *
	 * Widget styling
	 *
	 */
	public function print_header_styles() {
		?>
		<style type="text/css">
		.ttf-updater-password-input {
			display: block;
			position: relative;
		}

		.ttf-updater-password-input input {
			font-family: Consolas, Monaco, monospace;
			width: 100%;
		}

		button.ttf-updater-hide-pw {
			position: absolute;
			right: 0px;
			top: 1px;
			display: flex !important;
			justify-content: center;
			align-items: center;
			width: 35px;
			min-height: auto !important;
			height: calc(100% - 2px);
			padding: 0 !important;
			border: 1px solid transparent !important;
			border-radius: 3px;
			background: transparent !important;
		}

		button.ttf-updater-hide-pw:focus {
			background: transparent !important;
			border-color: #3582c4;
			box-shadow: 0 0 0 1px #3582c4 !important;
			outline: 2px solid transparent !important;
		}

		button.ttf-updater-hide-pw .dashicons {
			display: block;
			width: 18px;
			height: 18px;
			font-size: 18px;
		}
		</style>
		<?php
	}

	/**
	 *
	 * Basic widget Javascript submission
	 *
	 */
	public function print_footer_scripts() {
		?>
		<script type="text/javascript">
		( function( $ ) {

		$( function() {
			$( document ).on( 'submit', '.ttf-product-updater-form', function( e ) {
				e.preventDefault();

				var $form = $( e.target );

				$.post( ajaxurl, $form.serialize(), function( html ) {
					if ( ! html ) {
						return;
					}

					$( '#ttf-product-updater-form-container' ).replaceWith( html );
					$( '#ttf-product-updater-form-container input:not([type="hidden"])' ).trigger( 'focus' );
				} );

				$( '.ttf-product-updater-form input, .ttf-product-updater-form button' ).prop( 'disabled', true );
			} );

			$( document ).on( 'click', 'a.skip', function( e ) {
				e.preventDefault();

				$.post( ajaxurl, { action: '<?php echo $this->action_skip; ?>' }, function( html ) {
					if ( ! html ) {
						return;
					}

					$( '#ttf-product-updater-form-container' ).replaceWith( html );
					$( '#ttf-product-updater-form-container input:not([type="hidden"])' ).trigger( 'focus' );
				} );

				$( '.ttf-product-updater-form input, .ttf-product-updater-form button' ).prop( 'disabled', true );
			} );

			$( document ).on( 'click', 'button.cancel', function( e ) {
				e.preventDefault();

				$.post( ajaxurl, { action: '<?php echo $this->action_cancel; ?>' }, function( html ) {
					if ( ! html ) {
						return;
					}

					$( '#ttf-product-updater-form-container' ).replaceWith( html );
					$( '#ttf-product-updater-form-container input:not([type="hidden"])' ).trigger( 'focus' );
				} );

				$( '.ttf-product-updater-form input, .ttf-product-updater-form button' ).prop( 'disabled', true );
			} );

			$( document ).on( 'click', '.ttf-updater-hide-pw', function( e ) {
				e.preventDefault();

				var $button = $( e.target ).closest( 'button' );
				var $input  = $button.prev();
				var $icon   = $( '> .dashicons', $button );

				$input.trigger( 'click' );

				setTimeout( function() {
					$input.trigger( 'select' );
				}, 100 );

				var revealIconClass = 'dashicons-visibility';
				var hideIconClass   = 'dashicons-hidden';

				if ( 'password' === $input.attr( 'type' ) ) {
					$button.attr( 'aria-label', $button.attr( 'data-label-hide' ) );
					$input.attr( 'type', 'text' );
					$icon.removeClass( revealIconClass ).addClass( hideIconClass );
				} else {
					$button.attr( 'aria-label', $button.attr( 'data-label-show' ) );
					$input.attr( 'type', 'password' );
					$icon.removeClass( hideIconClass ).addClass( revealIconClass );
				}
			} );
		} );

		} )( jQuery );
		</script>
		<?php
	}

	/**
	 *
	 * Fetch a product feed.
	 *
	 */
	public function get_feed() {
		$response = wp_remote_get( $this->product->url_feed, array(
			'headers' => array(
				'X-TTF-PRODUCT-PLAN' => $this->product->plan,
			),
			'timeout' => 5,
		) );

		if ( is_wp_error( $response ) ) {
			return false;
		}

		$code = (int) wp_remote_retrieve_response_code( $response );

		if ( 200 !== $code ) {
			return false;
		}

		$body = json_decode( wp_remote_retrieve_body( $response ) );

		if ( 'success' !== $body->code ) {
			return false;
		}

		// Prepare feed data for consumption by core.
		$info = $body->data;
		$info->download_link = $info->package;
		$info->sections = (array) $info->sections;

		// Allow injecting icons from the plugin itself.
		$icons = isset( $info->icons ) ? (array) $info->icons : false;
		$icons = apply_filters( 'ttf_product_updater_icons', $icons, $this->product->id );

		if ( $icons ) {
			$info->icons = $icons;
		}

		// Allow injecting banners from the plugin itself.
		$banners = isset( $info->banners ) ? (array) $info->banners : false;
		$banners = apply_filters( 'ttf_product_updater_banners', $banners, $this->product->id );

		if ( $banners ) {
			$info->banners = $banners;
		}

		// Stub translation support.
		if ( isset( $info->translations ) ) {
			$info->translations = (array) $info->translations;
		}

		return $info;
	}

	/**
	 *
	 * Called during core update checks.
	 *
	 * Adds the current plugin to the updates transient,
	 * if a new version is available.
	 *
	 */
	public function pre_set_site_transient_update_plugins( $transient ) {
		if ( empty( $transient ) ) {
			return $transient;
		}

		if ( empty( $transient->checked ) ) {
			$transient->checked = array();
		}

		if ( 'plugin' !== $this->product->type ) {
			return $transient;
		}

		$feed = $this->get_feed();

		// Bail if, for some reason, a feed is unavailable.
		if ( ! $feed ) {
			return $transient;
		}

		// Add the plugin to the updates transient, if a new version is available.
		if ( version_compare( $this->product->version, $feed->version, '<' ) ) {
			$transient->response[$this->product->slug] = $this->get_update( $feed );
		}

		// Add the plugin to the list of plugins checked for updates.
		$transient->checked[$this->product->slug] = $this->product->version;

		return $transient;
	}

	/**
	 *
	 * Called during core update checks.
	 *
	 * Adds the current theme to the updates transient,
	 * if a new version is available.
	 *
	 */
	public function pre_set_site_transient_update_themes( $transient ) {
		if ( empty( $transient ) ) {
			return $transient;
		}

		if ( empty( $transient->checked ) ) {
			$transient->checked = array();
		}

		if ( 'theme' !== $this->product->type ) {
			return $transient;
		}

		$feed = $this->get_feed();

		// Bail if, for some reason, a feed is unavailable.
		if ( ! $feed ) {
			return $transient;
		}

		// Add the plugin to the updates transient, if a new version is available.
		if ( version_compare( $this->product->version, $feed->version, '<' ) ) {
			$transient->response[$this->product->slug] = array(
				'theme' => $this->product->slug,
				'new_version' => $feed->version,
				'url' => $feed->homepage,
				'package' => $feed->package,
			);
		}

		// Add the plugin to the list of plugins checked for updates.
		$transient->checked[$this->product->slug] = $this->product->version;

		return $transient;
	}

	/**
	 *
	 * Called from core's "View version details" links.
	 *
	 * Populates the update's details overlay.
	 *
	 */
	public function plugins_api( $false, $action, $args ) {
		if ( 'plugin_information' !== $action ) {
			return $false;
		}

		if ( $args->slug !== $this->product->id ) {
			return $false;
		}

		$info = $this->get_feed();

		if ( ! $info ) {
			return $false;
		}

		return $info;
	}

	/**
	 *
	 * Called from core's "View version details" links.
	 *
	 * Populates the update's details overlay.
	 *
	 */
	public function themes_api( $false, $action, $args ) {
		if ( 'theme_information' !== $action ) {
			return $false;
		}

		if ( $args->slug !== $this->product->id ) {
			return $false;
		}

		$info = $this->get_feed();

		if ( ! $info ) {
			return $false;
		}

		return $info;
	}

	/**
	 *
	 * Called by core's while it gathers update details
	 * before actually performing an update.
	 *
	 * Turns feed details into update details for core's upgrader.
	 *
	 */
	public function get_update( $update_info ) {
		$update = new stdClass();
		$update->slug = $this->product->id;
		$update->plugin = $this->product->slug;
		$update->new_version = $update_info->version;
		$update->url = $update_info->homepage;
		$update->package = $update_info->package;

		if ( isset( $update_info->tested ) ) {
			$update->tested = $update_info->tested;
		}

		if ( isset( $update_info->banners ) ) {
			$update->banners = $update_info->banners;
		}

		if ( isset( $update_info->icons ) ) {
			$update->icons = $update_info->icons;
		}

		if ( isset( $update_info->upgrade_notice ) ) {
			$update->upgrade_notice = $update_info->upgrade_notice;
		}

		return $update;
	}

	/**
	 *
	 * Called by core's upgrader when preparing a package for download.
	 *
	 * If the user has access to a download, this update package is turned
	 * into a download url, otherwise it becomes a placeholder used to display
	 * an expired license message later.
	 *
	 */
	public function upgrader_package_options( $options ) {
		if ( $this->product->url_package === $options['package'] ) {
			$license_key = $this->get_license_key();

			// The license key is missing.
			if ( empty( $license_key ) ) {
				$options['package'] = 'ttf-product-updater-unregistered-' . $this->product->id;

				return $options;
			}

			$response = wp_remote_get( $options['package'], array(
				'headers' => array(
					'X-TTF-LICENSE-KEY' => $this->get_license_key(),
					'X-TTF-PRODUCT-PLAN' => $this->product->plan,
					'X-TTF-PRODUCT-VERSION' => $this->product->version,
					'X-TTF-SITE-URL' => get_site_url(),
				),
				'timeout' => 5,
			) );

			// Default return package, if license checks don't pass.
			$options['package'] = 'ttf-product-updater-expired-' . $this->product->id;

			$code = (int) wp_remote_retrieve_response_code( $response );

			if ( 200 !== $code ) {
				return $options;
			}

			$body = json_decode( wp_remote_retrieve_body( $response ) );

			if ( 'success' !== $body->code ) {
				return $options;
			}

			// License checks passed, return the package download.
			$options['package'] = $this->product->url_download;
		}

		return $options;
	}

	/**
	 *
	 * Called by core's upgrader when downloading a package.
	 *
	 * If the user has access to a download, this lets the request
	 * pass through, otherwise it returns an expired license error.
	 *
	 */
	public function upgrader_pre_download( $reply, $package, $upgrader ) {
		if ( false !== $reply ) {
			return $reply;
		}

		if ( 0 === strpos( $package, 'ttf-product-updater-unregistered-' ) ) {
			return new WP_Error(
				'ttf_updater_license_missing',
				sprintf(
					__( 'Oops! Either your key is incorrect or your membership is deactivated.', 'ttf-product-updater' ),
					$this->product->name
				)
			);
		}

		if ( 0 === strpos( $package, 'ttf-product-updater-expired-' ) ) {
			return new WP_Error(
				'ttf_updater_license_expired',
				sprintf(
					__( 'Oops! Either your key is incorrect or your membership is deactivated.', 'ttf-product-updater' ),
					$this->product->name
				)
			);
		}

		return $reply;
	}

	/**
	 *
	 * Force requests to download URLs to include authentication headers.
	 *
	 */
	public function http_request_args( $args, $url ) {
		if ( $this->product->url_download === $url ) {
			$args['headers'] = wp_parse_args( $args['headers'], array(
				'X-TTF-LICENSE-KEY' => $this->get_license_key(),
				'X-TTF-PRODUCT-PLAN' => $this->product->plan,
				'X-TTF-PRODUCT-VERSION' => $this->product->version,
				'X-TTF-SITE-URL' => get_site_url(),
			) );
		}

		return $args;
	}

	/**
	 *
	 * Poll update server periodically and validate the license.
	 *
	 */
	public function license_check() {
		$response = wp_remote_get( $this->product->url_license_check, array(
			'headers' => array(
				'X-TTF-LICENSE-KEY' => $this->get_license_key(),
				'X-TTF-PRODUCT-PLAN' => $this->product->plan,
				'X-TTF-PRODUCT-VERSION' => $this->product->version,
				'X-TTF-SITE-URL' => get_site_url(),
			),
			'timeout' => 5,
		) );
	}

}

endif;
